@@ -235,6 +235,7 @@ gth_file_data_cmp_date_time_original (GthFileData *a,
 {
 	GTimeVal *pta, *ptb;
 	GTimeVal  ta, tb;
+	int       result;
 
 	pta = NULL;
 	if (gth_file_data_get_digitalization_time (a, &ta))
@@ -248,7 +249,17 @@ gth_file_data_cmp_date_time_original (GthFileData *a,
 	if (ptb == NULL)
 		ptb = gth_file_data_get_modification_time (b);
 
-	return _g_time_val_cmp (pta, ptb);
+	result = _g_time_val_cmp (pta, ptb);
+	if (result == 0) {
+		const char *key_a, *key_b;
+
+		key_a = gth_file_data_get_filename_sort_key (a);
+		key_b = gth_file_data_get_filename_sort_key (b);
+
+		result = strcmp (key_a, key_b);
+	}
+
+	return result;
 }
 
 
@@ -279,7 +290,7 @@ gthumb_extension_activate (void)
 		gth_main_register_type ("edit-comment-dialog-page", GTH_TYPE_EDIT_IPTC_PAGE);
 		gth_hook_add_callback ("delete-metadata", 10, G_CALLBACK (exiv2_delete_metadata_cb), NULL);
 	}
-	gth_hook_add_callback ("save-pixbuf", 10, G_CALLBACK (exiv2_write_metadata), NULL);
+	gth_hook_add_callback ("save-image", 10, G_CALLBACK (exiv2_write_metadata), NULL);
 	if (gth_hook_present ("jpegtran-after"))
 		gth_hook_add_callback ("jpegtran-after", 10, G_CALLBACK (exiv2_jpeg_tran_cb), NULL);
 	gth_hook_add_callback ("generate-thumbnail", 10, G_CALLBACK (exiv2_generate_thumbnail), NULL);
