@@ -239,7 +239,7 @@ update_image_surface (GthImageRotator *self)
 	gtk_widget_get_allocation (GTK_WIDGET (self->priv->viewer), &allocation);
 	max_size = MAX (allocation.width, allocation.height) / G_SQRT2 + 2;
 	if (scale_keeping_ratio (&width, &height, max_size, max_size, FALSE))
-		preview_image = _cairo_image_surface_scale_to (image, width, height, CAIRO_FILTER_BILINEAR);
+		preview_image = _cairo_image_surface_scale_bilinear (image, width, height);
 	else
 		preview_image = cairo_surface_reference (image);
 
@@ -304,49 +304,61 @@ paint_darker_background (GthImageRotator *self,
 			 cairo_t         *cr)
 {
 	cairo_rectangle_int_t crop_region;
+	GtkAllocation         allocation;
 
 	cairo_save (cr);
 	cairo_set_source_rgba (cr, 0.0, 0.0, 0.0, 0.5);
 
-	/* the crop_region is not zoomed the clip_area is already zoomed */
+	gtk_widget_get_allocation (GTK_WIDGET (self->priv->viewer), &allocation);
 
-	crop_region = self->priv->crop_region;
-	crop_region.x = crop_region.x * self->priv->preview_zoom;
-	crop_region.y = crop_region.y * self->priv->preview_zoom;
-	crop_region.width = crop_region.width * self->priv->preview_zoom;
-	crop_region.height = crop_region.height * self->priv->preview_zoom;
+	switch (self->priv->resize) {
+	case GTH_TRANSFORM_RESIZE_BOUNDING_BOX:
+	case GTH_TRANSFORM_RESIZE_CLIP:
+		crop_region = self->priv->clip_area;
+		break;
+
+	case GTH_TRANSFORM_RESIZE_CROP:
+		/* the crop_region is not zoomed the clip_area is already zoomed */
+		cairo_scale (cr, self->priv->preview_zoom, self->priv->preview_zoom);
+		crop_region = self->priv->crop_region;
+		crop_region.x += self->priv->clip_area.x / self->priv->preview_zoom;
+		crop_region.y += self->priv->clip_area.y / self->priv->preview_zoom;
+		allocation.width /= self->priv->preview_zoom;
+		allocation.height /= self->priv->preview_zoom;
+		break;
+	}
 
 	/* left side */
 
 	cairo_rectangle (cr,
-			 self->priv->clip_area.x,
-			 self->priv->clip_area.y,
+			 0,
+			 0,
 			 crop_region.x,
-			 self->priv->clip_area.height);
+			 allocation.height);
 
 	/* right side */
 
 	cairo_rectangle (cr,
-			 self->priv->clip_area.x + crop_region.x + crop_region.width,
-			 self->priv->clip_area.y,
-			 self->priv->clip_area.width - crop_region.x - crop_region.width,
-			 self->priv->clip_area.height);
+			 crop_region.x + crop_region.width,
+			 0,
+			 allocation.width - crop_region.x - crop_region.width,
+			 allocation.height);
 
 	/* top */
 
 	cairo_rectangle (cr,
-			 self->priv->clip_area.x,
-			 self->priv->clip_area.y,
-			 self->priv->clip_area.width,
+			 crop_region.x,
+			 0,
+			 crop_region.width,
 			 crop_region.y);
 
 	/* bottom */
 
 	cairo_rectangle (cr,
-			 self->priv->clip_area.x,
-			 self->priv->clip_area.y + crop_region.y + crop_region.height,
-			 self->priv->clip_area.width,
-			 self->priv->clip_area.height - crop_region.y - crop_region.height);
+			 crop_region.x,
+			 crop_region.y + crop_region.height,
+			 crop_region.width,
+			 allocation.height - crop_region.y - crop_region.height);
 
 	cairo_fill (cr);
 	cairo_restore (cr);
@@ -361,11 +373,20 @@ paint_grid (GthImageRotator *self,
 
 	cairo_save (cr);
 
-	cairo_scale (cr, self->priv->preview_zoom, self->priv->preview_zoom);
+	switch (self->priv->resize) {
+	case GTH_TRANSFORM_RESIZE_BOUNDING_BOX:
+	case GTH_TRANSFORM_RESIZE_CLIP:
+		grid = self->priv->clip_area;
+		break;
+
+	case GTH_TRANSFORM_RESIZE_CROP:
+		cairo_scale (cr, self->priv->preview_zoom, self->priv->preview_zoom);
+		grid = self->priv->crop_region;
+		grid.x += self->priv->clip_area.x / self->priv->preview_zoom;
+		grid.y += self->priv->clip_area.y / self->priv->preview_zoom;
+		break;
+	}
 
-	grid = self->priv->crop_region;
-	grid.x += self->priv->clip_area.x / self->priv->preview_zoom;
-	grid.y += self->priv->clip_area.y / self->priv->preview_zoom;
 	_cairo_paint_grid (cr, &grid, self->priv->grid_type);
 
 	cairo_restore (cr);
@@ -379,11 +400,19 @@ paint_point (GthImageRotator *self,
 {
 	double radius = 10.0;
 
+	cairo_save (cr);
+
+#if CAIRO_VERSION >= CAIRO_VERSION_ENCODE(1, 9, 2)
+	cairo_set_operator (cr, CAIRO_OPERATOR_DIFFERENCE);
+#endif
+
 	cairo_move_to (cr, p->x - radius, p->y - radius);
 	cairo_line_to (cr, p->x + radius, p->y + radius);
 	cairo_move_to (cr, p->x - radius, p->y + radius);
 	cairo_line_to (cr, p->x + radius, p->y - radius);
 	cairo_stroke (cr);
+
+	cairo_restore (cr);
 }
 
 
@@ -392,46 +421,38 @@ gth_image_rotator_draw (GthImageViewerTool *base,
 			cairo_t            *cr)
 {
 	GthImageRotator *self = GTH_IMAGE_ROTATOR (base);
-	GtkStyleContext *style_context;
-	GdkRGBA          color;
 	GtkAllocation    allocation;
 
 	cairo_save (cr);
 
   	/* background */
 
+	/*
+	GtkStyleContext *style_context;
+	GdkRGBA          color;
 	style_context = gtk_widget_get_style_context (GTK_WIDGET (self->priv->viewer));
 	gtk_style_context_get_background_color (style_context,
 						gtk_widget_get_state (GTK_WIDGET (self->priv->viewer)),
 						&color);
 	gdk_cairo_set_source_rgba (cr, &color);
+	*/
+
 	gtk_widget_get_allocation (GTK_WIDGET (self->priv->viewer), &allocation);
 	cairo_rectangle (cr, 0, 0, allocation.width, allocation.height);
-	cairo_fill (cr);
 
-	if (self->priv->preview_image == NULL)
-		return;
-
-	/* clip box */
-
-  	cairo_rectangle (cr,
-  			 self->priv->clip_area.x,
-  			 self->priv->clip_area.y,
-  			 self->priv->clip_area.width,
-  			 self->priv->clip_area.height);
-  	cairo_clip_preserve (cr);
   	cairo_set_source_rgba (cr,
   			       self->priv->background_color.r,
   			       self->priv->background_color.g,
   			       self->priv->background_color.b,
   			       self->priv->background_color.a);
-  	cairo_fill (cr);
+	cairo_fill (cr);
+
+	if (self->priv->preview_image == NULL)
+		return;
 
 	paint_image (self, cr);
-	if (self->priv->enable_crop) {
-		paint_darker_background (self, cr);
-		paint_grid (self, cr);
-	}
+	paint_darker_background (self, cr);
+	paint_grid (self, cr);
 
 	if (self->priv->dragging) {
 		GdkPoint center;
@@ -564,13 +585,13 @@ gth_image_rotator_motion_notify (GthImageViewerTool *base,
 
 		angle1 = get_angle (&center, &self->priv->drag_p1);
 		angle2 = get_angle (&center, &self->priv->drag_p2);
-		if (angle2 < angle1 - G_PI)
-			angle2 = G_2_PI + angle2;
-		if (angle2 > angle1 + G_PI)
-			angle2 = angle2 - G_2_PI;
 		angle = self->priv->angle_before_dragging + (angle2 - angle1);
+		if (angle <  - G_PI)
+			angle = G_2_PI + angle;
+		if (angle >  + G_PI)
+			angle = angle - G_2_PI;
 
-		g_signal_emit (self, signals[ANGLE_CHANGED], 0, CLAMP (RAD_TO_DEG (angle), -90.0, 90));
+		g_signal_emit (self, signals[ANGLE_CHANGED], 0, CLAMP (RAD_TO_DEG (angle), -180.0, 180));
 	}
 
 	return FALSE;
@@ -865,7 +886,9 @@ gth_image_rotator_get_result_fast (GthImageRotator *self)
 		self->priv->crop_region.height = clip_area.height;
 	}
 
-	output = cairo_image_surface_create (CAIRO_FORMAT_ARGB32, self->priv->crop_region.width, self->priv->crop_region.height);
+	output = cairo_image_surface_create (CAIRO_FORMAT_ARGB32,
+					     self->priv->crop_region.width,
+					     self->priv->crop_region.height);
 
 	/* set the device offset to make the clip area start from the top left
 	 * corner of the output image */
