@@ -417,7 +417,8 @@ photoset_combobox_changed_cb (GtkComboBox *widget,
 
 
 static GthImage *
-flickr_thumbnail_loader (GthFileData   *file_data,
+flickr_thumbnail_loader (GInputStream  *istream,
+			 GthFileData   *file_data,
 			 int            requested_size,
 			 int           *original_width,
 			 int           *original_height,
@@ -450,7 +451,7 @@ flickr_thumbnail_loader (GthFileData   *file_data,
 		gsize  size;
 
 		file = g_file_new_for_uri (uri);
-		if (g_load_file_in_buffer (file, &buffer, &size, cancellable, error)) {
+		if (_g_file_load_in_buffer (file, &buffer, &size, cancellable, error)) {
 			GInputStream *stream;
 			GdkPixbuf    *pixbuf;
 
