@@ -29,6 +29,7 @@
 #include <extensions/gstreamer_utils/gstreamer-utils.h>
 #include "actions.h"
 #include "gth-media-viewer-page.h"
+#include "preferences.h"
 
 
 #define GET_WIDGET(x) (_gtk_builder_get_widget (self->priv->builder, (x)))
@@ -87,7 +88,6 @@ static const char *media_viewer_ui_info =
 "<ui>"
 "  <toolbar name='ViewerToolBar'>"
 "    <placeholder name='ViewerCommands'>"
-"      <toolitem action='View_Fullscreen'/>"
 "      <toolitem action='MediaViewer_Screenshot'/>"
 "    </placeholder>"
 "  </toolbar>"
@@ -215,7 +215,7 @@ video_area_draw_cb (GtkWidget *widget,
 		if (size > allocation.height)
 			size = allocation.height;
 		size = size / 3;
-		self->priv->icon = _g_icon_get_pixbuf (icon, size, gtk_icon_theme_get_for_screen (gtk_widget_get_screen (widget)));
+		self->priv->icon = _g_icon_get_pixbuf (icon, size, _gtk_widget_get_icon_theme (widget));
 
 		g_object_unref (icon);
 		g_free (type);
@@ -319,8 +319,7 @@ static void position_value_changed_cb (GtkAdjustment *adjustment,
 
 
 static void
-update_current_position_bar (GthMediaViewerPage *self,
-			     gboolean            update_progressbar)
+update_current_position_bar (GthMediaViewerPage *self)
 {
 	GstFormat format;
         gint64    current_value = 0;
@@ -329,31 +328,29 @@ update_current_position_bar (GthMediaViewerPage *self,
         if (gst_element_query_position (self->priv->playbin, &format, &current_value)) {
         	char *s;
 
-        	if (self->priv->duration <= 0) {
-        		gst_element_query_duration (self->priv->playbin, &format, &self->priv->duration);
-        		s = _g_format_duration_for_display (GST_TIME_AS_MSECONDS (self->priv->duration));
-        		gtk_label_set_text (GTK_LABEL (GET_WIDGET ("label_duration")), s);
+		if (self->priv->duration <= 0) {
+			gst_element_query_duration (self->priv->playbin, &format, &self->priv->duration);
+			s = _g_format_duration_for_display (GST_TIME_AS_MSECONDS (self->priv->duration));
+			gtk_label_set_text (GTK_LABEL (GET_WIDGET ("label_duration")), s);
 
-        		g_free (s);
-        	}
+			g_free (s);
+		}
 
-        	/*
-        	g_print ("==> %" G_GINT64_FORMAT " / %" G_GINT64_FORMAT " (%0.3g)\n" ,
-        		 current_value,
-        		 self->priv->duration,
-        		 ((double) current_value / self->priv->duration) * 100.0);
+		/*
+		g_print ("==> %" G_GINT64_FORMAT " / %" G_GINT64_FORMAT " (%0.3g)\n" ,
+			 current_value,
+			 self->priv->duration,
+			 ((double) current_value / self->priv->duration) * 100.0);
 		*/
 
-        	if (update_progressbar) {
-			g_signal_handlers_block_by_func(GET_WIDGET ("position_adjustment"), position_value_changed_cb, self);
-			gtk_adjustment_set_value (GTK_ADJUSTMENT (GET_WIDGET ("position_adjustment")), (self->priv->duration > 0) ? ((double) current_value / self->priv->duration) * 100.0 : 0.0);
-			g_signal_handlers_unblock_by_func(GET_WIDGET ("position_adjustment"), position_value_changed_cb, self);
-        	}
+		g_signal_handlers_block_by_func(GET_WIDGET ("position_adjustment"), position_value_changed_cb, self);
+		gtk_adjustment_set_value (GTK_ADJUSTMENT (GET_WIDGET ("position_adjustment")), (self->priv->duration > 0) ? ((double) current_value / self->priv->duration) * 100.0 : 0.0);
+		g_signal_handlers_unblock_by_func(GET_WIDGET ("position_adjustment"), position_value_changed_cb, self);
 
-        	s = _g_format_duration_for_display (GST_TIME_AS_MSECONDS (current_value));
-        	gtk_label_set_text (GTK_LABEL (GET_WIDGET ("label_position")), s);
+		s = _g_format_duration_for_display (GST_TIME_AS_MSECONDS (current_value));
+		gtk_label_set_text (GTK_LABEL (GET_WIDGET ("label_position")), s);
 
-        	g_free (s);
+		g_free (s);
         }
 }
 
@@ -628,9 +625,13 @@ update_volume_from_playbin (GthMediaViewerPage *self)
 	else
 		gtk_image_set_from_icon_name (GTK_IMAGE (GET_WIDGET ("volume_togglebutton_image")), "audio-volume-high-symbolic", GTK_ICON_SIZE_BUTTON);
 
-	g_signal_handlers_block_by_func(GET_WIDGET ("volume_adjustment"), volume_value_changed_cb, self);
+	g_signal_handlers_block_by_func (GET_WIDGET ("volume_adjustment"), volume_value_changed_cb, self);
 	gtk_adjustment_set_value (GTK_ADJUSTMENT (GET_WIDGET ("volume_adjustment")), volume * 100.0);
-	g_signal_handlers_unblock_by_func(GET_WIDGET ("volume_adjustment"), volume_value_changed_cb, self);
+	g_signal_handlers_unblock_by_func (GET_WIDGET ("volume_adjustment"), volume_value_changed_cb, self);
+
+	g_signal_handlers_block_by_func (GET_WIDGET ("volume_togglebutton"), volume_togglebutton_toggled_cb, self);
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (GET_WIDGET ("volume_togglebutton")), volume == 0.0);
+	g_signal_handlers_unblock_by_func (GET_WIDGET ("volume_togglebutton"), volume_togglebutton_toggled_cb, self);
 }
 
 
@@ -644,7 +645,7 @@ update_progress_cb (gpointer user_data)
                 self->priv->update_progress_id = 0;
         }
 
-        update_current_position_bar (self, TRUE);
+        update_current_position_bar (self);
 
         self->priv->update_progress_id = gdk_threads_add_timeout (PROGRESS_DELAY, update_progress_cb, self);
 
@@ -659,7 +660,7 @@ set_playing_state (GthMediaViewerPage *self,
 	self->priv->playing = playing;
 	if (self->priv->playing)
 		gth_screensaver_inhibit (self->priv->screensaver,
-					 GTK_WIDGET (self->priv->browser),
+					 GTK_WINDOW (self->priv->browser),
 					 _("Playing video"));
 	else
 		gth_screensaver_uninhibit (self->priv->screensaver);
@@ -840,6 +841,20 @@ remove_fullscreen_toolbar (GthMediaViewerPage *self)
 }
 
 
+static void
+save_volume (GthMediaViewerPage *self)
+{
+	GSettings *settings;
+	double     volume;
+
+	settings = g_settings_new (GTHUMB_GSTREAMER_TOOLS_SCHEMA);
+	g_object_get (self->priv->playbin, "volume", &volume, NULL);
+	g_settings_set_int (settings, PREF_GSTREAMER_TOOLS_VOLUME, (int) (volume * 100.0));
+
+	g_object_unref (settings);
+}
+
+
 static void
 gth_media_viewer_page_real_deactivate (GthViewerPage *base)
 {
@@ -858,6 +873,7 @@ gth_media_viewer_page_real_deactivate (GthViewerPage *base)
         }
 
 	if (self->priv->playbin != NULL) {
+		save_volume (self);
 		gst_element_set_state (self->priv->playbin, GST_STATE_NULL);
 		gst_object_unref (GST_OBJECT (self->priv->playbin));
 		self->priv->playbin = NULL;
@@ -1004,7 +1020,7 @@ bus_message_cb (GstBus     *bus,
 		if (GST_MESSAGE_SRC (message) != GST_OBJECT (self->priv->playbin))
 			break;
 
-		update_current_position_bar (self, TRUE);
+		update_current_position_bar (self);
 
 		if ((old_state == GST_STATE_NULL) && (new_state == GST_STATE_READY) && (pending_state != GST_STATE_PAUSED)) {
 			update_stream_info (self);
@@ -1023,15 +1039,6 @@ bus_message_cb (GstBus     *bus,
 		break;
 	}
 
-	case GST_MESSAGE_DURATION: {
-		GstFormat format;
-
-		format = GST_FORMAT_TIME;
-		gst_message_parse_duration (message, &format, &self->priv->duration);
-		update_current_position_bar (self, TRUE);
-		break;
-	}
-
 	case GST_MESSAGE_EOS:
 		reset_player_state (self);
 		break;
@@ -1065,12 +1072,21 @@ playbin_notify_volume_cb (GObject    *playbin,
 static void
 create_playbin (GthMediaViewerPage *self)
 {
-	GstBus *bus;
+	GSettings *settings;
+	GstBus    *bus;
 
 	if (self->priv->playbin != NULL)
 		return;
 
 	self->priv->playbin = gst_element_factory_make ("playbin", "playbin");
+
+	settings = g_settings_new (GTHUMB_GSTREAMER_TOOLS_SCHEMA);
+	g_object_set (self->priv->playbin,
+		      "volume",
+		      (double) g_settings_get_int (settings, PREF_GSTREAMER_TOOLS_VOLUME) / 100.0,
+		      NULL);
+	g_object_unref (settings);
+
 	g_signal_connect (self->priv->playbin, "notify::volume", G_CALLBACK (playbin_notify_volume_cb), self);
 
 	bus = gst_pipeline_get_bus (GST_PIPELINE (self->priv->playbin));
@@ -1393,6 +1409,7 @@ gth_media_viewer_page_finalize (GObject *obj)
         }
 
 	if (self->priv->playbin != NULL) {
+		save_volume (self);
 		gst_element_set_state (self->priv->playbin, GST_STATE_NULL);
 		gst_object_unref (GST_OBJECT (self->priv->playbin));
 		self->priv->playbin = NULL;
