@@ -631,7 +631,7 @@ paint_comment_over_image_func (GthImageViewer *image_viewer,
         	GIcon *gicon;
 
         	gicon = g_themed_icon_new (GTK_STOCK_PROPERTIES);
-        	icon = _g_icon_get_pixbuf (gicon, 24, NULL);
+        	icon = _g_icon_get_pixbuf (gicon, 24, _gtk_widget_get_icon_theme (GTK_WIDGET (image_viewer)));
 
         	g_object_unref (gicon);
         }
@@ -693,7 +693,6 @@ gth_image_viewer_page_real_activate (GthViewerPage *base,
 				     GthBrowser    *browser)
 {
 	GthImageViewerPage *self;
-	GtkAction          *action;
 
 	self = (GthImageViewerPage*) base;
 
@@ -707,9 +706,6 @@ gth_image_viewer_page_real_activate (GthViewerPage *base,
 				      self);
 	gtk_ui_manager_insert_action_group (gth_browser_get_ui_manager (browser), self->priv->actions, 0);
 
-	action = gtk_action_group_get_action (gth_browser_get_actions (browser), "Viewer_Tools");
-	g_object_set (action, "sensitive", TRUE, NULL);
-
 	self->priv->preloader = gth_browser_get_image_preloader (browser);
 	self->priv->requested_ready_id = g_signal_connect (G_OBJECT (self->priv->preloader),
 							   "requested_ready",
@@ -814,13 +810,9 @@ static void
 gth_image_viewer_page_real_deactivate (GthViewerPage *base)
 {
 	GthImageViewerPage *self;
-	GtkAction          *action;
 
 	self = (GthImageViewerPage*) base;
 
-	action = gtk_action_group_get_action (gth_browser_get_actions (self->priv->browser), "Viewer_Tools");
-	g_object_set (action, "sensitive", FALSE, NULL);
-
 	if (self->priv->browser_merge_id != 0) {
 		gtk_ui_manager_remove_ui (gth_browser_get_ui_manager (self->priv->browser), self->priv->browser_merge_id);
 		self->priv->browser_merge_id = 0;
@@ -1108,7 +1100,7 @@ _gth_image_viewer_page_real_save (GthViewerPage *base,
 	GthImageViewerPage *self;
 	SaveData           *data;
 	GthFileData        *current_file;
-	GdkPixbuf          *pixbuf;
+	GthImage           *image;
 
 	self = (GthImageViewerPage *) base;
 
@@ -1147,15 +1139,15 @@ _gth_image_viewer_page_real_save (GthViewerPage *base,
 	 * wants to save (see load_file_delayed_cb in gth-browser.c). */
 	g_file_info_set_attribute_boolean (data->file_to_save->info, "gth::file::is-modified", FALSE);
 
-	pixbuf = gth_image_viewer_get_current_pixbuf (GTH_IMAGE_VIEWER (self->priv->viewer));
-	_gdk_pixbuf_save_async (pixbuf,
+	image = gth_image_new_for_surface (gth_image_viewer_get_current_image (GTH_IMAGE_VIEWER (self->priv->viewer)));
+	gth_image_save_to_file (image,
+				mime_type,
 				data->file_to_save,
-			        mime_type,
 			        TRUE,
 				image_saved_cb,
 				data);
 
-	_g_object_unref (pixbuf);
+	_g_object_unref (image);
 }
 
 
@@ -1164,7 +1156,7 @@ gth_image_viewer_page_real_can_save (GthViewerPage *base)
 {
 	GArray *savers;
 
-	savers = gth_main_get_type_set ("pixbuf-saver");
+	savers = gth_main_get_type_set ("image-saver");
 	return (savers != NULL) && (savers->len > 0);
 }
 
@@ -1222,12 +1214,15 @@ save_as_response_cb (GtkDialog  *file_sel,
 	if (! gth_file_chooser_dialog_get_file (GTH_FILE_CHOOSER_DIALOG (file_sel), &file, &mime_type))
 		return;
 
+	gtk_widget_hide (GTK_WIDGET (data->file_sel));
+
 	gth_file_data_set_file (data->file_data, file);
 	_gth_image_viewer_page_real_save ((GthViewerPage *) data->self,
 					  file,
 					  mime_type,
 					  data->func,
 					  data->user_data);
+
 	gtk_widget_destroy (GTK_WIDGET (data->file_sel));
 
 	g_object_unref (file);
@@ -1247,7 +1242,7 @@ gth_image_viewer_page_real_save_as (GthViewerPage *base,
 	self = GTH_IMAGE_VIEWER_PAGE (base);
 	file_sel = gth_file_chooser_dialog_new (_("Save Image"),
 						GTK_WINDOW (self->priv->browser),
-						"pixbuf-saver");
+						"image-saver");
 
 	uri = g_file_get_uri (self->priv->file_data->file);
 	gtk_file_chooser_set_uri (GTK_FILE_CHOOSER (file_sel), uri);
