@@ -334,7 +334,7 @@ after_saving_to_destination (GthImportTask  *self,
 		if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_EXISTS)) {
 			if (self->priv->default_response != GTH_OVERWRITE_RESPONSE_ALWAYS_NO) {
 				GInputStream *stream;
-				GdkPixbuf    *pixbuf;
+				GthImage     *image;
 				GtkWidget    *dialog;
 
 				/* take ownership of the buffer */
@@ -353,15 +353,15 @@ after_saving_to_destination (GthImportTask  *self,
 
 				if (self->priv->buffer != NULL) {
 					stream = g_memory_input_stream_new_from_data (self->priv->buffer, self->priv->buffer_size, NULL);
-					pixbuf = gdk_pixbuf_new_from_stream_at_scale (stream, 128, 128, TRUE, NULL, NULL);
+					image = gth_image_new_from_stream (stream, 128, NULL, NULL, NULL, NULL);
 				}
 				else {
 					stream = NULL;
-					pixbuf = NULL;
+					image = NULL;
 				}
 
 				dialog = gth_overwrite_dialog_new (file_data->file,
-								   pixbuf,
+								   image,
 								   self->priv->destination_file->file,
 								   self->priv->default_response,
 								   self->priv->files->next == NULL);
@@ -372,7 +372,7 @@ after_saving_to_destination (GthImportTask  *self,
 				gtk_widget_show (dialog);
 				gth_task_dialog (GTH_TASK (self), TRUE, dialog);
 
-				_g_object_unref (pixbuf);
+				_g_object_unref (image);
 				_g_object_unref (stream);
 			}
 			else
@@ -482,16 +482,16 @@ write_file_to_destination (GthImportTask *self,
 				   FALSE,
 				   (double) (self->priv->copied_size + ((double) self->priv->current_file_size / 3.0 * 2.0)) / self->priv->tot_size);
 
-		self->priv->buffer = NULL; /* the buffer will be deallocated in g_write_file_async */
+		self->priv->buffer = NULL; /* the buffer will be deallocated in _g_file_write_async */
 
-		g_write_file_async (self->priv->destination_file->file,
-				    buffer,
-				    count,
-				    replace,
-				    G_PRIORITY_DEFAULT,
-				    gth_task_get_cancellable (GTH_TASK (self)),
-				    write_buffer_ready_cb,
-				    self);
+		_g_file_write_async (self->priv->destination_file->file,
+				     buffer,
+				     count,
+				     replace,
+				     G_PRIORITY_DEFAULT,
+				     gth_task_get_cancellable (GTH_TASK (self)),
+				     write_buffer_ready_cb,
+				     self);
 	}
 	else
 		g_file_copy_async (file_data->file,
@@ -575,7 +575,7 @@ file_buffer_ready_cb (void     **buffer,
 				   *buffer,
 				   count,
 				   self->priv->default_response == GTH_OVERWRITE_RESPONSE_ALWAYS_YES);
-	*buffer = NULL; /* g_write_file_async takes ownership of the buffer */
+	*buffer = NULL; /* _g_file_write_async takes ownership of the buffer */
 
 	g_object_unref (destination_file);
 }
@@ -653,11 +653,11 @@ import_current_file (GthImportTask *self)
 				   FALSE,
 				   (double) (self->priv->copied_size + ((double) self->priv->current_file_size / 3.0)) / self->priv->tot_size);
 
-		g_load_file_async (file_data->file,
-				   G_PRIORITY_DEFAULT,
-				   gth_task_get_cancellable (GTH_TASK (self)),
-				   file_buffer_ready_cb,
-				   self);
+		_g_file_load_async (file_data->file,
+				    G_PRIORITY_DEFAULT,
+				    gth_task_get_cancellable (GTH_TASK (self)),
+				    file_buffer_ready_cb,
+				    self);
 	}
 	else {
 		GFile *destination_file;
