@@ -34,6 +34,8 @@ G_DEFINE_TYPE (PicasaWebService, picasa_web_service, G_TYPE_OBJECT)
 typedef struct {
 	PicasaWebAlbum      *album;
 	GList               *file_list;
+	int                  max_width;
+	int                  max_height;
 	GCancellable        *cancellable;
         GAsyncReadyCallback  callback;
         gpointer             user_data;
@@ -439,6 +441,8 @@ post_photo_file_buffer_ready_cb (void     **buffer,
 	gsize               entry_len;
 	SoupMessageHeaders *headers;
 	SoupBuffer         *body;
+	void               *resized_buffer;
+	gsize               resized_count;
 	char               *url;
 	SoupMessage        *msg;
 
@@ -507,7 +511,27 @@ post_photo_file_buffer_ready_cb (void     **buffer,
 
 	/* the file part */
 
-	body = soup_buffer_new (SOUP_MEMORY_TEMPORARY, *buffer, count);
+	if (_g_buffer_resize_image (*buffer,
+				    count,
+				    file_data,
+				    self->priv->post_photos->max_width,
+				    self->priv->post_photos->max_height,
+				    &resized_buffer,
+				    &resized_count,
+				    self->priv->post_photos->cancellable,
+				    &error))
+	{
+		body = soup_buffer_new (SOUP_MEMORY_TAKE, resized_buffer, resized_count);
+	}
+	else if (error == NULL) {
+		body = soup_buffer_new (SOUP_MEMORY_TEMPORARY, *buffer, count);
+	}
+	else {
+		soup_multipart_free (multipart);
+		post_photos_done (self, error);
+		return;
+	}
+
 	soup_multipart_append_form_file (multipart,
 					 "file",
 					 NULL,
@@ -555,11 +579,11 @@ picasa_wev_service_post_current_file (PicasaWebService *self)
 	}
 
 	file_data = self->priv->post_photos->current->data;
-	g_load_file_async (file_data->file,
-			   G_PRIORITY_DEFAULT,
-			   self->priv->post_photos->cancellable,
-			   post_photo_file_buffer_ready_cb,
-			   self);
+	_g_file_load_async (file_data->file,
+			    G_PRIORITY_DEFAULT,
+			    self->priv->post_photos->cancellable,
+			    post_photo_file_buffer_ready_cb,
+			    self);
 }
 
 
@@ -595,6 +619,8 @@ void
 picasa_web_service_post_photos (PicasaWebService    *self,
 			        PicasaWebAlbum      *album,
 			        GList               *file_list, /* GFile list */
+			        int                  max_width,
+			        int                  max_height,
 			        GCancellable        *cancellable,
 			        GAsyncReadyCallback  callback,
 			        gpointer             user_data)
@@ -606,6 +632,8 @@ picasa_web_service_post_photos (PicasaWebService    *self,
 
 	self->priv->post_photos = g_new0 (PostPhotosData, 1);
 	self->priv->post_photos->album = g_object_ref (album);
+	self->priv->post_photos->max_width = max_width;
+	self->priv->post_photos->max_height = max_height;
 	self->priv->post_photos->cancellable = _g_object_ref (cancellable);
 	self->priv->post_photos->callback = callback;
 	self->priv->post_photos->user_data = user_data;
@@ -755,14 +783,14 @@ GList *
 picasa_web_accounts_load_from_file (char **_default)
 {
 	GList       *accounts = NULL;
-	char        *filename;
+	GFile       *file;
 	char        *buffer;
 	gsize        len;
 	DomDocument *doc;
 
-	filename = gth_user_dir_get_file (GTH_DIR_CONFIG, GTHUMB_DIR, "accounts", "picasaweb.xml", NULL);
-	if (! g_file_get_contents (filename, &buffer, &len, NULL)) {
-		g_free (filename);
+	file = gth_user_dir_get_file_for_read (GTH_DIR_CONFIG, GTHUMB_DIR, "accounts", "picasaweb.xml", NULL);
+	if (! _g_file_load_in_buffer (file, (void **) &buffer, &len, NULL, NULL)) {
+		g_object_unref (file);
 		return NULL;
 	}
 
@@ -795,7 +823,7 @@ picasa_web_accounts_load_from_file (char **_default)
 
 	g_object_unref (doc);
 	g_free (buffer);
-	g_free (filename);
+	g_object_unref (file);
 
 	return accounts;
 }
@@ -810,7 +838,6 @@ picasa_web_accounts_save_to_file (GList      *accounts,
 	GList       *scan;
 	char        *buffer;
 	gsize        len;
-	char        *filename;
 	GFile       *file;
 
 	doc = dom_document_new ();
@@ -828,14 +855,11 @@ picasa_web_accounts_save_to_file (GList      *accounts,
 		dom_element_append_child (root, node);
 	}
 
-	gth_user_dir_make_dir_for_file (GTH_DIR_CONFIG, GTHUMB_DIR, "accounts", "picasaweb.xml", NULL);
-	filename = gth_user_dir_get_file (GTH_DIR_CONFIG, GTHUMB_DIR, "accounts", "picasaweb.xml", NULL);
-	file = g_file_new_for_path (filename);
+	file = gth_user_dir_get_file_for_write (GTH_DIR_CONFIG, GTHUMB_DIR, "accounts", "picasaweb.xml", NULL);
 	buffer = dom_document_dump (doc, &len);
-	g_write_file (file, FALSE, G_FILE_CREATE_PRIVATE | G_FILE_CREATE_REPLACE_DESTINATION, buffer, len, NULL, NULL);
+	_g_file_write (file, FALSE, G_FILE_CREATE_PRIVATE | G_FILE_CREATE_REPLACE_DESTINATION, buffer, len, NULL, NULL);
 
 	g_free (buffer);
 	g_object_unref (file);
-	g_free (filename);
 	g_object_unref (doc);
 }
