@@ -93,6 +93,11 @@ _gdk_color_to_cairo_color_255 (GdkColor          *g_color,
 void
 _cairo_clear_surface (cairo_surface_t  **surface)
 {
+	if (surface == NULL)
+		return;
+	if (*surface == NULL)
+		return;
+
 	cairo_surface_destroy (*surface);
 	*surface = NULL;
 }
@@ -340,29 +345,6 @@ _cairo_image_surface_create_compatible (cairo_surface_t *surface)
 }
 
 
-cairo_surface_t *
-_cairo_image_surface_scale_to (cairo_surface_t *surface,
-			       int              width,
-			       int              height,
-			       cairo_filter_t   filter)
-{
-	cairo_surface_t *scaled;
-	cairo_t         *cr;
-
-	scaled = cairo_image_surface_create (CAIRO_FORMAT_ARGB32, width, height);
-	cr = cairo_create (scaled);
-	cairo_scale (cr, (double) width / cairo_image_surface_get_width (surface), (double) height / cairo_image_surface_get_height (surface));
-	cairo_set_source_surface (cr, surface, 0, 0);
-	cairo_pattern_set_filter (cairo_get_source (cr), filter);
-	cairo_rectangle (cr, 0, 0, cairo_image_surface_get_width (surface), cairo_image_surface_get_height (surface));
-	cairo_fill (cr);
-
-	cairo_destroy (cr);
-
-	return scaled;
-}
-
-
 void
 _cairo_image_surface_transform_get_steps (cairo_format_t  format,
 					  int             width,
@@ -529,6 +511,88 @@ _cairo_image_surface_transform (cairo_surface_t *source,
 }
 
 
+cairo_surface_t *
+_cairo_image_surface_color_shift (cairo_surface_t *image,
+				  int              shift)
+{
+	cairo_surface_t *shifted;
+	int              i, j;
+	int              width, height, src_stride, dest_stride;
+	guchar          *src_pixels, *src_row, *src_pixel;
+	guchar          *dest_pixels, *dest_row, *dest_pixel;
+	int              val;
+	guchar           r, g, b, a;
+
+	shifted = _cairo_image_surface_create_compatible (image);
+
+	width       = cairo_image_surface_get_width (image);
+	height      = cairo_image_surface_get_height (image);
+	src_stride  = cairo_image_surface_get_stride (image);
+	src_pixels  = cairo_image_surface_get_data (image);
+	dest_stride = cairo_image_surface_get_stride (shifted);
+	dest_pixels = cairo_image_surface_get_data (shifted);
+
+	src_row = src_pixels;
+	dest_row = dest_pixels;
+	for (i = 0; i < height; i++) {
+		src_pixel = src_row;
+		dest_pixel  = dest_row;
+
+		for (j = 0; j < width; j++) {
+			CAIRO_GET_RGBA (src_pixel, r, g, b, a);
+
+			val = r + shift;
+			r = CLAMP (val, 0, 255);
+
+			val = g + shift;
+			g = CLAMP (val, 0, 255);
+
+			val = b + shift;
+			b = CLAMP (val, 0, 255);
+
+			CAIRO_SET_RGBA (dest_pixel, r, g, b, a);
+
+			src_pixel += 4;
+			dest_pixel += 4;
+		}
+
+		src_row += src_stride;
+		dest_row += dest_stride;
+	}
+
+	return shifted;
+}
+
+
+void
+_cairo_copy_line_as_rgba (guchar *dest,
+			  guchar *src,
+			  guint   width,
+			  guint   alpha)
+{
+	guint x;
+
+	if (alpha) {
+		for (x = 0; x < width; x++) {
+			CAIRO_GET_RGBA (src, dest[0], dest[1], dest[2], dest[3]);
+
+			src += 4;
+			dest += 4;
+		}
+	}
+	else {
+		for (x = 0; x < width; x++) {
+			dest[0] = src[CAIRO_RED];
+			dest[1] = src[CAIRO_GREEN];
+			dest[2] = src[CAIRO_BLUE];
+
+			src += 4;
+			dest += 3;
+		}
+	}
+}
+
+
 void
 _cairo_paint_full_gradient (cairo_surface_t *surface,
 			    GdkColor        *h_color1,
@@ -780,6 +844,7 @@ _cairo_draw_slide (cairo_t  *cr,
 #define GRID_STEP_1         10
 #define GRID_STEP_2         (GRID_STEP_1 * 5)
 #define GRID_STEP_3         (GRID_STEP_2 * 2)
+#define MAX_GRID_LINES      50
 
 
 void
@@ -799,7 +864,7 @@ _cairo_paint_grid (cairo_t               *cr,
 	cairo_clip (cr);
 
 #if CAIRO_VERSION >= CAIRO_VERSION_ENCODE(1, 9, 2)
-	/* cairo_set_operator (cr, CAIRO_OPERATOR_DIFFERENCE); */
+	cairo_set_operator (cr, CAIRO_OPERATOR_DIFFERENCE);
 #endif
 
 	cairo_rectangle (cr, rectangle->x + 0.5, rectangle->y + 0.5, rectangle->width - 0.5, rectangle->height - 0.5);
@@ -811,8 +876,6 @@ _cairo_paint_grid (cairo_t               *cr,
 		return;
 	}
 
-	cairo_set_operator (cr, CAIRO_OPERATOR_OVER);
-
 	if (grid_type == GTH_GRID_THIRDS) {
 		int i;
 
@@ -907,44 +970,50 @@ _cairo_paint_grid (cairo_t               *cr,
 		int x;
 		int y;
 
-		cairo_set_source_rgba (cr, 1.0, 1.0, 1.0, 0.40);
-		for (x = GRID_STEP_3; x < rectangle->width; x += GRID_STEP_3) {
-			cairo_move_to (cr, rectangle->x + x + 0.5, rectangle->y + 1.5);
-			cairo_line_to (cr, rectangle->x + x + 0.5, rectangle->y + rectangle->height - 0.5);
+		if (rectangle->width / GRID_STEP_3 <= MAX_GRID_LINES) {
+			cairo_set_source_rgba (cr, 1.0, 1.0, 1.0, 0.40);
+			for (x = GRID_STEP_3; x < rectangle->width; x += GRID_STEP_3) {
+				cairo_move_to (cr, rectangle->x + x + 0.5, rectangle->y + 1.5);
+				cairo_line_to (cr, rectangle->x + x + 0.5, rectangle->y + rectangle->height - 0.5);
+			}
+			for (y = GRID_STEP_3; y < rectangle->height; y += GRID_STEP_3) {
+				cairo_move_to (cr, rectangle->x + 1.5, rectangle->y + y + 0.5);
+				cairo_line_to (cr, rectangle->x + rectangle->width - 0.5, rectangle->y + y + 0.5);
+			}
+			cairo_stroke (cr);
 		}
-		for (y = GRID_STEP_3; y < rectangle->height; y += GRID_STEP_3) {
-			cairo_move_to (cr, rectangle->x + 1.5, rectangle->y + y + 0.5);
-			cairo_line_to (cr, rectangle->x + rectangle->width - 0.5, rectangle->y + y + 0.5);
-		}
-		cairo_stroke (cr);
 
-		cairo_set_source_rgba (cr, 1.0, 1.0, 1.0, 0.20);
-		for (x = GRID_STEP_2; x < rectangle->width; x += GRID_STEP_2) {
-			if (x % GRID_STEP_3 == 0)
-				continue;
-			cairo_move_to (cr, rectangle->x + x + 0.5, rectangle->y + 1.5);
-			cairo_line_to (cr, rectangle->x + x + 0.5, rectangle->y + rectangle->height - 0.5);
+		if (rectangle->width / GRID_STEP_2 <= MAX_GRID_LINES) {
+			cairo_set_source_rgba (cr, 1.0, 1.0, 1.0, 0.20);
+			for (x = GRID_STEP_2; x < rectangle->width; x += GRID_STEP_2) {
+				if (x % GRID_STEP_3 == 0)
+					continue;
+				cairo_move_to (cr, rectangle->x + x + 0.5, rectangle->y + 1.5);
+				cairo_line_to (cr, rectangle->x + x + 0.5, rectangle->y + rectangle->height - 0.5);
+			}
+			for (y = GRID_STEP_2; y < rectangle->height; y += GRID_STEP_2) {
+				if (y % GRID_STEP_3 == 0)
+					continue;
+				cairo_move_to (cr, rectangle->x + 1.5, rectangle->y + y + 0.5);
+				cairo_line_to (cr, rectangle->x + rectangle->width - 0.5, rectangle->y + y + 0.5);
+			}
+			cairo_stroke (cr);
 		}
-		for (y = GRID_STEP_2; y < rectangle->height; y += GRID_STEP_2) {
-			if (y % GRID_STEP_3 == 0)
-				continue;
-			cairo_move_to (cr, rectangle->x + 1.5, rectangle->y + y + 0.5);
-			cairo_line_to (cr, rectangle->x + rectangle->width - 0.5, rectangle->y + y + 0.5);
-		}
-		cairo_stroke (cr);
 
-		cairo_set_source_rgba (cr, 1.0, 1.0, 1.0, 0.10);
-		for (x = GRID_STEP_1; x < rectangle->width; x += GRID_STEP_1) {
-			if (x % GRID_STEP_2 == 0)
-				continue;
-			cairo_move_to (cr, rectangle->x + x + 0.5, rectangle->y + 1.5);
-			cairo_line_to (cr, rectangle->x + x + 0.5, rectangle->y + rectangle->height - 0.5);
-		}
-		for (y = GRID_STEP_1; y < rectangle->height; y += GRID_STEP_1) {
-			if (y % GRID_STEP_2 == 0)
-				continue;
-			cairo_move_to (cr, rectangle->x + 1.5, rectangle->y + y + 0.5);
-			cairo_line_to (cr, rectangle->x + rectangle->width - 0.5, rectangle->y + y + 0.5);
+		if (rectangle->width / GRID_STEP_1 <= MAX_GRID_LINES) {
+			cairo_set_source_rgba (cr, 1.0, 1.0, 1.0, 0.10);
+			for (x = GRID_STEP_1; x < rectangle->width; x += GRID_STEP_1) {
+				if (x % GRID_STEP_2 == 0)
+					continue;
+				cairo_move_to (cr, rectangle->x + x + 0.5, rectangle->y + 1.5);
+				cairo_line_to (cr, rectangle->x + x + 0.5, rectangle->y + rectangle->height - 0.5);
+			}
+			for (y = GRID_STEP_1; y < rectangle->height; y += GRID_STEP_1) {
+				if (y % GRID_STEP_2 == 0)
+					continue;
+				cairo_move_to (cr, rectangle->x + 1.5, rectangle->y + y + 0.5);
+				cairo_line_to (cr, rectangle->x + rectangle->width - 0.5, rectangle->y + y + 0.5);
+			}
 		}
 		cairo_stroke (cr);
 	}
