@@ -37,6 +37,7 @@ G_BEGIN_DECLS
 #define GTHUMB_DIALOGS_SCHEMA                 GTHUMB_SCHEMA ".dialogs"
 #define GTHUMB_MESSAGES_SCHEMA                GTHUMB_DIALOGS_SCHEMA ".messages"
 #define GTHUMB_ADD_TO_CATALOG_SCHEMA          GTHUMB_DIALOGS_SCHEMA ".add-to-catalog"
+#define GTHUMB_SAVE_FILE_SCHEMA               GTHUMB_DIALOGS_SCHEMA ".save-file"
 
 /* keys: general */
 
@@ -67,6 +68,7 @@ G_BEGIN_DECLS
 #define PREF_BROWSER_TOOLBAR_STYLE            "toolbar-style"
 #define PREF_BROWSER_WINDOW_WIDTH             "window-width"
 #define PREF_BROWSER_WINDOW_HEIGHT            "window-height"
+#define PREF_BROWSER_WINDOW_MAXIMIZED         "maximized"
 #define PREF_BROWSER_TOOLBAR_VISIBLE          "toolbar-visible"
 #define PREF_BROWSER_STATUSBAR_VISIBLE        "statusbar-visible"
 #define PREF_BROWSER_FILTERBAR_VISIBLE        "filterbar-visible"
@@ -75,6 +77,7 @@ G_BEGIN_DECLS
 #define PREF_BROWSER_PROPERTIES_ON_THE_RIGHT  "properties-on-the-right"
 #define PREF_BROWSER_THUMBNAIL_LIST_VISIBLE   "thumbnail-list-visible"
 #define PREF_BROWSER_BROWSER_SIDEBAR_WIDTH    "browser-sidebar-width"
+#define PREF_BROWSER_VIEWER_SIDEBAR           "viewer-sidebar"
 #define PREF_BROWSER_VIEWER_SIDEBAR_WIDTH     "viewer-sidebar-width"
 #define PREF_BROWSER_VIEWER_THUMBNAILS_ORIENT "viewer-thumbnails-orientation"
 #define PREF_BROWSER_SHRINK_WRAP_VIEWER       "shrink-wrap-viewer"
@@ -85,6 +88,10 @@ G_BEGIN_DECLS
 #define PREF_ADD_TO_CATALOG_LAST_CATALOG      "last-catalog"
 #define PREF_ADD_TO_CATALOG_VIEW              "view"
 
+/* keys: save file */
+
+#define PREF_SAVE_FILE_SHOW_OPTIONS           "show-options"
+
 /* keys: messages */
 
 #define PREF_MSG_CANNOT_MOVE_TO_TRASH         "cannot-move-to-trash"
