@@ -35,6 +35,7 @@
 #include "gnome-desktop-thumbnail.h"
 #include "gth-error.h"
 #include "gth-image-loader.h"
+#include "gth-image-utils.h"
 #include "gth-main.h"
 #include "gth-thumb-loader.h"
 #include "pixbuf-io.h"
@@ -111,7 +112,8 @@ gth_thumb_loader_init (GthThumbLoader *self)
 
 
 static GthImage *
-generate_thumbnail (GthFileData   *file_data,
+generate_thumbnail (GInputStream  *istream,
+		    GthFileData   *file_data,
 		    int            requested_size,
 		    int           *original_width,
 		    int           *original_height,
@@ -121,8 +123,15 @@ generate_thumbnail (GthFileData   *file_data,
 {
 	GthThumbLoader *self = user_data;
 	GdkPixbuf      *pixbuf = NULL;
-	GthImage       *image;
+	GthImage       *image = NULL;
 	char           *uri;
+	const char     *mime_type;
+
+	if (file_data == NULL) {
+		if (error != NULL)
+			*error = g_error_new_literal (G_IO_ERROR, G_IO_ERROR_INVALID_FILENAME, "Could not load file");
+		return NULL;
+	}
 
 	if (original_width != NULL)
 		*original_width = -1;
@@ -130,26 +139,45 @@ generate_thumbnail (GthFileData   *file_data,
 	if (original_height != NULL)
 		*original_height = -1;
 
-	image = NULL;
+	mime_type = _g_content_type_get_from_stream (istream, file_data->file, cancellable, error);
+	if (mime_type == NULL) {
+		if ((error != NULL) && (*error == NULL))
+			*error = g_error_new_literal (GTH_ERROR, 0, "Cannot generate the thumbnail: unknown file type");
+		return NULL;
+	}
+
 	uri = g_file_get_uri (file_data->file);
 	pixbuf = gnome_desktop_thumbnail_factory_generate_no_script (self->priv->thumb_factory,
 								     uri,
-								     gth_file_data_get_mime_type (file_data),
+								     mime_type,
 								     cancellable);
 
 	if (g_cancellable_is_cancelled (cancellable)) {
 		_g_object_unref (pixbuf);
+		g_free (uri);
 		if (error != NULL)
 			*error = g_error_new_literal (G_IO_ERROR, G_IO_ERROR_CANCELLED, "");
 		return NULL;
 	}
 
-	if (pixbuf == NULL) {
+	if (pixbuf != NULL) {
+		image = gth_image_new_for_pixbuf (pixbuf);
+		if (original_width != NULL)
+			*original_width = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (pixbuf), "gnome-original-width"));
+		if (original_height != NULL)
+			*original_height = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (pixbuf), "gnome-original-height"));
+		if (error != NULL)
+			g_clear_error (error);
+
+		g_object_unref (pixbuf);
+	}
+	else {
 		GthImageLoaderFunc thumbnailer;
 
-		thumbnailer = gth_main_get_image_loader_func (gth_file_data_get_mime_type (file_data), GTH_IMAGE_FORMAT_GDK_PIXBUF);
+		thumbnailer = gth_main_get_image_loader_func (mime_type, GTH_IMAGE_FORMAT_GDK_PIXBUF);
 		if (thumbnailer != NULL)
-			image = thumbnailer (file_data,
+			image = thumbnailer (istream,
+					     file_data,
 					     self->priv->cache_max_size,
 					     original_width,
 					     original_height,
@@ -158,23 +186,8 @@ generate_thumbnail (GthFileData   *file_data,
 					     error);
 	}
 
-	if (pixbuf != NULL) {
-		if (original_width != NULL)
-			*original_width = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (pixbuf), "gnome-original-width"));
-		if (original_height != NULL)
-			*original_height = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (pixbuf), "gnome-original-height"));
-
-		if (error != NULL)
-			g_clear_error (error);
-
-		image = gth_image_new_for_pixbuf (pixbuf);
-
-		g_object_unref (pixbuf);
-	}
-
-	if (image == NULL)
-		if (error != NULL)
-			*error = g_error_new_literal (GTH_ERROR, 0, "Cannot generate the thumbnail");
+	if ((image == NULL) && (error != NULL))
+		*error = g_error_new_literal (GTH_ERROR, 0, "Could not generate the thumbnail");
 
 	g_free (uri);
 
@@ -183,7 +196,8 @@ generate_thumbnail (GthFileData   *file_data,
 
 
 static GthImage *
-load_cached_thumbnail (GthFileData   *file_data,
+load_cached_thumbnail (GInputStream  *istream,
+		       GthFileData   *file_data,
 		       int            requested_size,
 		       int           *original_width,
 		       int           *original_height,
@@ -195,6 +209,12 @@ load_cached_thumbnail (GthFileData   *file_data,
 	char      *filename;
 	GdkPixbuf *pixbuf;
 
+	if (file_data == NULL) {
+		if (error != NULL)
+			*error = g_error_new_literal (G_IO_ERROR, G_IO_ERROR_INVALID_FILENAME, "Could not load file");
+		return NULL;
+	}
+
 	filename = g_file_get_path (file_data->file);
 	pixbuf = gdk_pixbuf_new_from_file (filename, error);
 	if (pixbuf != NULL) {
@@ -444,7 +464,7 @@ cache_image_ready_cb (GObject      *source_object,
 				    self->priv->cache_max_size);
 	if (modified) {
 		GdkPixbuf *tmp = pixbuf;
-		pixbuf = gdk_pixbuf_scale_simple (tmp, width, height, GDK_INTERP_BILINEAR);
+		pixbuf = _gdk_pixbuf_scale_simple_safe (tmp, width, height, GDK_INTERP_BILINEAR);
 		g_object_unref (tmp);
 	}
 
@@ -462,13 +482,21 @@ cache_image_ready_cb (GObject      *source_object,
 static gboolean
 is_a_cache_file (const char *uri)
 {
-	char     *cache_base_uri;
+	char     *filename;
+	char     *cache_dir_1;
+	char     *cache_dir_2;
 	gboolean  result;
 
-	cache_base_uri = g_strconcat (get_home_uri (), "/.thumbnails", NULL);
-	result = _g_uri_parent_of_uri (cache_base_uri, uri);
+	filename = g_filename_from_uri (uri, NULL, NULL);
+	if (filename == NULL)
+		return FALSE;
 
-	g_free (cache_base_uri);
+	cache_dir_1 = g_build_filename (g_get_home_dir (), ".thumbnails", NULL);
+	cache_dir_2 = g_build_filename (g_get_user_cache_dir (), "thumbnails", NULL);
+	result = _g_uri_parent_of_uri (cache_dir_1, filename) || _g_uri_parent_of_uri (cache_dir_2, filename);
+
+	g_free (cache_dir_1);
+	g_free (cache_dir_2);
 
 	return result;
 }
@@ -480,50 +508,24 @@ _gth_thumb_loader_save_to_cache (GthThumbLoader *self,
 				 GdkPixbuf      *pixbuf)
 {
 	char  *uri;
-	char  *cache_path;
-	GFile *cache_file;
-	GFile *cache_dir;
 
 	if ((self == NULL) || (pixbuf == NULL))
 		return FALSE;
 
 	uri = g_file_get_uri (file_data->file);
 
-	if (g_file_is_native (file_data->file)) {
+	/* Do not save thumbnails from the user's thumbnail directory,
+	 * or an endless loop of thumbnailing may be triggered. */
 
-		/* Do not save thumbnails from the user's thumbnail directory,
-		   or an endless loop of thumbnailing may be triggered. */
-
-		if (is_a_cache_file (uri)) {
-			g_free (uri);
-			return FALSE;
-		}
-	}
-
-	cache_path = gnome_desktop_thumbnail_path_for_uri (uri, self->priv->thumb_size);
-	if (cache_path == NULL) {
+	if (is_a_cache_file (uri)) {
 		g_free (uri);
 		return FALSE;
 	}
 
-	cache_file = g_file_new_for_path (cache_path);
-	cache_dir = g_file_get_parent (cache_file);
-
-	if (_g_directory_make (cache_dir, THUMBNAIL_DIR_PERMISSIONS, NULL)) {
-		char *uri;
-
-		uri = g_file_get_uri (file_data->file);
-		gnome_desktop_thumbnail_factory_save_thumbnail (self->priv->thumb_factory,
-								pixbuf,
-								uri,
-								gth_file_data_get_mtime (file_data));
-		g_free (uri);
-	}
-
-	g_object_unref (cache_dir);
-	g_object_unref (cache_file);
-	g_free (cache_path);
-	g_free (uri);
+	gnome_desktop_thumbnail_factory_save_thumbnail (self->priv->thumb_factory,
+							pixbuf,
+							uri,
+							gth_file_data_get_mtime (file_data));
 
 	return TRUE;
 }
@@ -574,7 +576,7 @@ original_image_loaded_correctly (GthThumbLoader *self,
 				    self->priv->cache_max_size);
 	if (modified) {
 		GdkPixbuf *tmp = local_pixbuf;
-		local_pixbuf = gdk_pixbuf_scale_simple (tmp, width, height, GDK_INTERP_BILINEAR);
+		local_pixbuf = _gdk_pixbuf_scale_simple_safe (tmp, width, height, GDK_INTERP_BILINEAR);
 		g_object_unref (tmp);
 	}
 
