@@ -21,8 +21,9 @@
 
 #include <config.h>
 #include <string.h>
+#include "gth-image-utils.h"
 #include "gtk-utils.h"
-#include "pixbuf-utils.h"
+
 
 #define REQUEST_ENTRY_WIDTH_IN_CHARS 40
 
@@ -417,93 +418,6 @@ _gtk_info_dialog_run (GtkWindow        *parent,
 }
 
 
-static GdkPixbuf *
-get_themed_icon_pixbuf (GThemedIcon  *icon,
-			int           size,
-			GtkIconTheme *icon_theme)
-{
-	char        **icon_names;
-	GtkIconInfo  *icon_info;
-	GdkPixbuf    *pixbuf;
-	GError       *error = NULL;
-
-	g_object_get (icon, "names", &icon_names, NULL);
-
-	if (icon_theme == NULL)
-		icon_theme = gtk_icon_theme_get_default ();
-	icon_info = gtk_icon_theme_choose_icon (icon_theme, (const char **)icon_names, size, 0);
-	if (icon_info == NULL)
-		icon_info = gtk_icon_theme_lookup_icon (icon_theme, "text-x-generic", size, GTK_ICON_LOOKUP_USE_BUILTIN);
-
-	pixbuf = gtk_icon_info_load_icon (icon_info, &error);
-	if (pixbuf == NULL) {
-		g_warning ("could not load icon pixbuf: %s\n", error->message);
-		g_clear_error (&error);
-	}
-
-	gtk_icon_info_free (icon_info);
-	g_strfreev (icon_names);
-
-	return pixbuf;
-}
-
-
-static GdkPixbuf *
-get_file_icon_pixbuf (GFileIcon *icon,
-		      int        size)
-{
-	GdkPixbuf *pixbuf = NULL;
-	GFile     *file;
-
-	file = g_file_icon_get_file (icon);
-	if (file != NULL) {
-		char *filename;
-
-		filename = g_file_get_path (file);
-		if (filename != NULL)
-			pixbuf = gdk_pixbuf_new_from_file_at_size (filename, size, -1, NULL);
-
-		g_free (filename);
-	}
-
-	return pixbuf;
-}
-
-
-GdkPixbuf *
-_g_icon_get_pixbuf (GIcon        *icon,
-		    int           size,
-		    GtkIconTheme *theme)
-{
-	GdkPixbuf *pixbuf;
-	int        w, h;
-
-	if (icon == NULL)
-		return NULL;
-
-	pixbuf = NULL;
-	if (G_IS_THEMED_ICON (icon))
-		pixbuf = get_themed_icon_pixbuf (G_THEMED_ICON (icon), size, theme);
-	else if (G_IS_FILE_ICON (icon))
-		pixbuf = get_file_icon_pixbuf (G_FILE_ICON (icon), size);
-
-	if (pixbuf == NULL)
-		return NULL;
-
-	w = gdk_pixbuf_get_width (pixbuf);
-	h = gdk_pixbuf_get_height (pixbuf);
-	if (scale_keeping_ratio (&w, &h, size, size, FALSE)) {
-		GdkPixbuf *scaled;
-
-		scaled = gdk_pixbuf_scale_simple (pixbuf, w, h, GDK_INTERP_BILINEAR);
-		g_object_unref (pixbuf);
-		pixbuf = scaled;
-	}
-
-	return pixbuf;
-}
-
-
 void
 _gtk_dialog_add_to_window_group (GtkDialog *dialog)
 {
@@ -517,6 +431,38 @@ _gtk_dialog_add_to_window_group (GtkDialog *dialog)
 }
 
 
+GdkPixbuf *
+_g_icon_get_pixbuf (GIcon        *icon,
+		    int           icon_size,
+		    GtkIconTheme *icon_theme)
+{
+	GdkPixbuf   *pixbuf = NULL;
+	GtkIconInfo *icon_info;
+
+	if (icon_theme == NULL)
+		icon_theme = gtk_icon_theme_get_default ();
+
+	icon_info = gtk_icon_theme_lookup_by_gicon (icon_theme,
+						    icon,
+						    icon_size,
+						    GTK_ICON_LOOKUP_USE_BUILTIN);
+
+	if (icon_info != NULL) {
+		GError *error = NULL;
+
+		pixbuf = gtk_icon_info_load_icon (icon_info, &error);
+		if (error != NULL) {
+			g_print ("%s\n", error->message);
+			g_error_free (error);
+		}
+
+		gtk_icon_info_free (icon_info);
+	}
+
+	return pixbuf;
+}
+
+
 GdkPixbuf *
 get_mime_type_pixbuf (const char   *mime_type,
 		      int           icon_size,
@@ -536,19 +482,6 @@ get_mime_type_pixbuf (const char   *mime_type,
 }
 
 
-int
-_gtk_icon_get_pixel_size (GtkWidget   *widget,
-			  GtkIconSize  size)
-{
-	int icon_width, icon_height;
-
-	gtk_icon_size_lookup_for_settings (gtk_widget_get_settings (widget),
-					   size,
-					   &icon_width, &icon_height);
-	return MAX (icon_width, icon_height);
-}
-
-
 void
 show_help_dialog (GtkWindow  *parent,
 		  const char *section)
@@ -805,6 +738,18 @@ _gtk_widget_get_allocated_height (GtkWidget *widget)
 }
 
 
+int
+_gtk_widget_lookup_for_size (GtkWidget   *widget,
+			     GtkIconSize  icon_size)
+{
+	int w, h;
+	gtk_icon_size_lookup_for_settings (gtk_widget_get_settings (widget),
+					   icon_size,
+					   &w, &h);
+	return MAX (w, h);
+}
+
+
 void
 _gtk_tree_path_list_free (GList *list)
 {
@@ -1036,47 +981,6 @@ _gtk_menu_ask_drag_drop_action (GtkWidget     *widget,
 }
 
 
-GtkAdjustment *
-gimp_scale_entry_new (GtkWidget *parent_box,
-		      GtkLabel  *label,
-		      float      value,
-		      float      lower,
-		      float      upper,
-		      float      step_increment,
-		      float      page_increment,
-		      int        digits)
-{
-	GtkAdjustment *adj;
-	GtkWidget     *spinbutton;
-	GtkWidget     *scale;
-	GtkWidget     *hbox;
-
-	adj = gtk_adjustment_new (value, lower, upper,
-				  step_increment, page_increment,
-				  0.0);
-
-	spinbutton = gtk_spin_button_new (GTK_ADJUSTMENT (adj), page_increment, 0);
-	gtk_spin_button_set_digits (GTK_SPIN_BUTTON (spinbutton), digits);
-	gtk_entry_set_width_chars (GTK_ENTRY (spinbutton), 4);
-
-	scale = gtk_scale_new (GTK_ORIENTATION_HORIZONTAL, GTK_ADJUSTMENT (adj));
-	gtk_scale_set_draw_value (GTK_SCALE (scale), FALSE);
-	gtk_scale_set_digits (GTK_SCALE (scale), digits);
-
-	hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 5);
-	gtk_box_pack_start (GTK_BOX (hbox), scale, TRUE, TRUE, 0);
-	gtk_box_pack_start (GTK_BOX (hbox), spinbutton, FALSE, FALSE, 0);
-
-	gtk_box_pack_start (GTK_BOX (parent_box), hbox, TRUE, TRUE, 0);
-	gtk_widget_show_all (hbox);
-
-	if (label != NULL)
-		gtk_label_set_mnemonic_widget (label, scale);
-
-	return (GtkAdjustment *) adj;
-}
-
-
 static gboolean
 _gdk_rgba_shade (GdkRGBA *color,
 		 GdkRGBA *result,
@@ -1112,3 +1016,48 @@ _gdk_rgba_lighter (GdkRGBA *color,
 {
 	return _gdk_rgba_shade (color, result, 1.2);
 }
+
+
+GtkIconTheme *
+_gtk_widget_get_icon_theme (GtkWidget *widget)
+{
+	GdkScreen *screen;
+
+	screen = gtk_widget_get_screen (widget);
+	if (screen == NULL)
+		return NULL;
+
+	return gtk_icon_theme_get_for_screen (screen);
+}
+
+
+void
+_gtk_combo_box_add_image_sizes (GtkComboBox *combo_box,
+				int          active_width,
+				int          active_height)
+{
+	GtkListStore *list_store;
+	int           active_index;
+	int           i;
+
+	list_store = GTK_LIST_STORE (gtk_combo_box_get_model (combo_box));
+	active_index = 0;
+	for (i = 0; i < G_N_ELEMENTS (ImageSizeValues); i++) {
+		GtkTreeIter  iter;
+		char        *name;
+
+		gtk_list_store_append (list_store, &iter);
+
+		if ((ImageSizeValues[i].width == active_width) && (ImageSizeValues[i].height == active_height))
+			active_index = i;
+
+		/* Translators: this is an image size, such as 1024 × 768 */
+		name = g_strdup_printf (_("%d × %d"), ImageSizeValues[i].width, ImageSizeValues[i].height);
+		gtk_list_store_set (list_store, &iter,
+				    0, name,
+				    -1);
+
+		g_free (name);
+	}
+	gtk_combo_box_set_active (combo_box, active_index);
+}
