@@ -81,13 +81,11 @@ void            _gtk_info_dialog_run                       (GtkWindow        *pa
 					    	    	    ...) G_GNUC_PRINTF (2, 3);
 void            _gtk_dialog_add_to_window_group            (GtkDialog        *dialog);
 GdkPixbuf *     _g_icon_get_pixbuf                         (GIcon            *icon,
-		 			    	    	    int               size,
+		 			    	    	    int               icon_size,
 		 			    	    	    GtkIconTheme     *icon_theme);
 GdkPixbuf *     get_mime_type_pixbuf                       (const char       *mime_type,
 					    	    	    int               icon_size,
 					    	    	    GtkIconTheme     *icon_theme);
-int             _gtk_icon_get_pixel_size                   (GtkWidget        *widget,
-					    	    	    GtkIconSize       size);
 void            show_help_dialog                           (GtkWindow        *parent,
 					    	    	    const char       *section);
 void            _gtk_container_remove_children             (GtkContainer     *container,
@@ -112,6 +110,8 @@ void            _gtk_widget_get_screen_size                (GtkWidget        *wi
 					    	    	    int              *height);
 int             _gtk_widget_get_allocated_width            (GtkWidget        *widget);
 int             _gtk_widget_get_allocated_height           (GtkWidget        *widget);
+int             _gtk_widget_lookup_for_size                (GtkWidget        *widget,
+							    GtkIconSize       icon_size);
 void            _gtk_tree_path_list_free                   (GList            *list);
 int             _gtk_paned_get_position2                   (GtkPaned         *paned);
 void            _gtk_paned_set_position2                   (GtkPaned         *paned,
@@ -126,18 +126,14 @@ void            _gtk_info_bar_clear_action_area            (GtkInfoBar       *in
 GdkDragAction   _gtk_menu_ask_drag_drop_action             (GtkWidget        *widget,
 					      	      	    GdkDragAction     actions,
 					      	      	    guint32           activate_time);
-GtkAdjustment * gimp_scale_entry_new        		   (GtkWidget        *parent_box,
-		      	      	      	      	      	    GtkLabel         *label,
-		      	      	      	      	      	    float             value,
-		      	      	      	      	      	    float             lower,
-		      	      	      	      	      	    float             upper,
-		      	      	      	      	      	    float             step_increment,
-		      	      	      	      	      	    float             page_increment,
-		      	      	      	      	      	    int               digits);
 gboolean        _gdk_rgba_darker                           (GdkRGBA          *color,
 		 					    GdkRGBA          *result);
 gboolean        _gdk_rgba_lighter                          (GdkRGBA          *color,
 							    GdkRGBA          *result);
+GtkIconTheme *  _gtk_widget_get_icon_theme                 (GtkWidget        *widget);
+void            _gtk_combo_box_add_image_sizes             (GtkComboBox      *combo_box,
+							    int               active_width,
+							    int               active_height);
 
 G_END_DECLS
 
